PRO azim_pow_params, ODIR, file, peaks, dk, df

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; .r /data/seismo1/schunker/IDL_PRO/mpfit.pro
; .r /data/seismo1/schunker/IDL_PRO/mpfitfun.pro
; .r /data/seismo1/schunker/IDL_PRO/mpfitpeak.pro
; 
;
; PURPOSE: reads in azimuthally averaged power spectra
; and for each k fit an asymmetric lorentzian to the power
; averaged profile. See Schunker et al (2010) for details
;
; INPUT:   ODIR - the directory where the power spectrum is and where
;                 the output will be stored.
;          file - name of the azimuthally averaged power spectrum
;          peaks - how many modes to fit for 
;          dk - scale in k-space - per Mm  (x-axis) 
;          df - scale in frequency space - per second (y-axis)
;
; Output: "fit_parameters.fits" in the ODIR, which are the fitting
; parameters 
;            ell = params[0]
;            f = params[ 1:peaks]         ; frequency (micro-Hz)
;            g = params[peaks+1:  2*peaks]    ; linewidth  (micro-Hz)
;            a = params[2*peaks+1:3*peaks]    ; mode power
;            b = params[3*peaks+1:4*peaks]    ; asymmetry parameter
;            n = params[4*peaks+1]*1.d0       ; background noise
;   
; WARNING: there are two options in this code, one to make a crude
; guess of the eigenfrequencies from the input, OR to take the guess
; from Model S. If you decide to use Model S guesses, you need to
; modify the directory where you keep the guesses. It is "MDIR" in
; this code.
;
;
;
; AUTHOR: Hannah Schunker
; schunker [@] mps.mpg.de
;
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; EXAMPLE
;azim_pow_params,"/data/seismo/schunker/SLIM/v2.5/RANDOM_QS_ATM1/RANDOM4/","powerspectra_Vz_8hrs.fits",5,0.0068601221,1.73611e-05 
;azim_pow_params,"/data/seismo2/moradi/final_csma_z300/","csma_power.fits",5,0.0068601221,1.73611e-05*1.6
;azim_pow_params,"/scratch/seismo1/schunker/final_csma_z300/","powerspectra_Vz_8hrs.fits",5,0.0068601221,1.73611e-05
;azim_pow_params,"/scratch/seismo1/schunker/final_csma_z300/","powerspectra_general.fits",5,0.0068601221,1.15741e-05


fname='powerspectra_analysis.ps'


;;;;;;;;;;     END CHANGES    ;;;;;;;;;;;



; read in the azimuthally averaged power spectra
power=readfits(ODIR+file,h)


nx=sxpar(h,'NAXIS1')
ny=sxpar(h,'NAXIS2')


; define ell
; ASSUMES ell BEGINS AT 0
Rs=696.d0   ; Mm
ell=findgen(nx)*dk*Rs*2.*!dpi
dell=ell(1)-ell(0)

; define frequency
freq=findgen(ny)*df*1d6   ; micro-Hz



; We normalise here to help with the fitting. It is still only a
; scaling by a number, but the fit is MUCH, MUCH BETTER if this step
; is done, although could probably be tweaked to work without it.

print, "  "
print, " NORMALISING  "
print, "  "
Rs=696.d8 ; cm
csb=5.86269e+06 ; cm /sec
rt=22.6*1e8  ; cm
; the formula is from JCD oscillation notes page 79, equation 5.28
psbb=csb*ell*1d8/(Rs-rt)/(2.*!dpi)*1e3   ; Hz

; mask for allowable region defined by the phase speed at the bottom
; of the box and below 1 mHz
ng1=power*0.
for i=0,nx-1 do ng1(i,*)=freq lt psbb(i)*1d3 and freq gt 1d3

power=power/max(power*ng1)



;;;;;;;;;   USING MODEL S FREQS AS A GUESS  ;;;;;;;;;;
if peaks gt 5 then begin
    print, "Do not have enough guesses from Model S"
    print, "for more than the p4 ridge. Exiting"
    GOTO, ENDED
endif


; to make guesses for location of frequency
; read in Model S eigenfrequencies
; CHANGE THIS DIRECTORY TO POINT TO WHERE YOU KEEP YOUR 
; MODEL S GUESSES
MDIR="BOX_145.770Mm/"
eigxl=readfits(MDIR+"harmonic_degree.fits")*1d0
feig=readfits(MDIR+"f_eigvals.fits")/(2*!pi)*1d6   ;micro Hz
peig1=readfits(MDIR+"p1_eigvals.fits")/(2*!pi)*1d6
peig2=readfits(MDIR+"p2_eigvals.fits")/(2*!pi)*1d6
peig3=readfits(MDIR+"p3_eigvals.fits")/(2*!pi)*1d6
peig4=readfits(MDIR+"p4_eigvals.fits")/(2*!pi)*1d6
eigxl=[0,reform(eigxl)]
feig=[feig(0),reform(feig)]
peig1=[peig1(0),reform(peig1)]
peig2=[peig2(0),reform(peig2)]
peig3=[peig3(0),reform(peig3)]
peig4=[peig4(0),reform(peig4)]


; image the powerspectra and overplot the model S guesses to make sure
; they will be in the ball park
loadct,0
!p.multi=[0,1,1,0,0]
imdisp,alog(power > 1e-3),aspect=1,xrange=[min(ell)-dell/2,max(ell)-dell/2],yrange=[0-df*1e6,max(freq)-df*1e6]*1e-3,/axis
oplot,eigxl,feig*1e-3
oplot,eigxl,peig1*1e-3
oplot,eigxl,peig2*1e-3
oplot,eigxl,peig3*1e-3
oplot,eigxl,peig4*1e-3


;;;;;;;;;;;;   END MODEL S GUESSING   ;;;;;;;;;;;;;;;;;;;;


; COLOUR FOR PLOTS
loadct,0
tvlct,r,g,b,/GET
blue=3
r(blue)= 0 & g(blue)=0 & b(blue)=255
tvlct,r,g,b



; set up array to store the parameters and the errors
param_array=fltarr(nx-0,4*peaks+2)
errors=fltarr(nx-0,4*peaks+1)

index=0.
; loop over ell indices
for l=20,20 do begin    


; select the power cut at constant ell 
    y=reform(power(l,*))

; a little smoothing 
    smooth_fac=2
    y=smooth(y,2)

; set the boundaries of the region to fit in frequency space
    if peaks eq 1 then begin
       cut_index1=0
       cut_index2=ny-1
       fg=freq(where(y eq max(y)))
    endif else begin
; lower boundary
        cut_index1=where(abs(freq-feig(l)) eq min(abs(freq-feig(l))))-(12+l*2)
; top boundary
        cut_index2=where(abs(freq-peig4(l)) eq min(abs(freq-peig4(l))))+(15+l*0.3)


; SOMETIMES THE TOP AND BOTTOM BOUNDARIES NEED TO BE ADJUSTED TO FIT
; IN THE SIZE OF THE PARTICULAR SPECTRA
; In fact, in retrospect, this is a dumb way to do this. The
; adjustment at the end should be interms of the frequency not l....



;;;;;;;;;  SET UP FREQUENCY GUESSES  ;;;;;;;;;  

; OPTION 1
; either use "find_guesses" function which uses the input
; power spectrum to find the peaks
; fg=find_guesses(freq(cut_index1:cut_index2),y(cut_index1:cut_index2),peaks)


; OPTION2
; or use Model S guesses
        fg=[feig(l),peig1(l),peig2(l),peig3(l),peig4(l)]

    endelse


; set up initial guess for all parameters
        guess=dblarr(4*peaks+1)
        guess(0:peaks-1)=fg                    ; frequencies
        guess(peaks:2*peaks-1)=250.            ; linewidth
        guess(2*peaks:3*peaks-1)=0.1           ; power
        if peaks eq 5 then guess(3*peaks:4*peaks-1)=0.    ; asymmetry
        if peaks eq 1 then guess(3*peaks:4*peaks-1)=0.1
        guess(4*peaks)=0.                    ; noise



; do a check
    if n_elements(guess) ne (peaks*4+1) then begin
        print, "Number of guesses does not match the number of peaks. Exiting."
        GOTO, ENDED
    endif


;;;;;;;;;; set up parameter information ;;;;;;;;;;;;;;
; (which to remain fixed, which to limit etc)
      pi = replicate({fixed:0, limited:[0,0], limits:[0.D,0.D]},21)
; limit the f-mode to lie within 1% of the guess, because we know the
; guess is good
      pi(0).limited=1.
      pi(0).limits(0)=fg(0)-fg(0)/100.
      pi(0).limits(1)=fg(0)+fg(0)/100.
      ;pi(4).limited=1.
      ;pi(4).limits(0)=fg(4)-5.*fg(4)/100.
      ;pi(4).limits(1)=fg(4)+5.*fg(4)/100.

    
      ; set weights evenly
    weights=fltarr(n_elements(y(cut_index1:cut_index2)))+1.

;;;;;;;;;; BEGIN FIT ;;;;;;;;;;;;;;

    result=mpfitfun('LORENTZIAN_ASYM_F',freq(cut_index1:cut_index2),y(cut_index1:cut_index2),weights,parinfo=pi,guess,/quiet)


; limit the frequencies, linewidth and power to be positive
    pi(0:peaks*3-1).limited=1.
    pi(0:peaks*3-1).limits(0)=0.
    pi(15:19).limited(0)=1
    pi(15:19).limited(1)=1 
 

;;;;;;;;;; FIT AGAIN ;;;;;;;;;;;;;;
    guess1=result 
    result=0
    result=mpfitfun('LORENTZIAN_ASYM_F',freq(cut_index1:cut_index2),y(cut_index1:cut_index2),weights,guess1,perror=errs,parinfo=pi,/QUIET)


    plot,freq(cut_index1:cut_index2),lorentzian_asym(freq(cut_index1:cut_index2),result),linestyle=3,thick=1

; NOTE: you can play around with how many times the fit is done, and
; which parameters are fixed, limited or free. 


;;; CHECK FITS AS IT GOES
    plot,freq(cut_index1:cut_index2),smooth(y(cut_index1:cut_index2),smooth_fac),xtit='frequency (Hz)',xst=1,charsize=1.5,ytickn=[' ',' ',' ',' ',' ',' ',' '],charthick=2,thick=1
    oplot,freq(cut_index1:cut_index2),lorentzian_asym(freq(cut_index1:cut_index2),result),linestyle=3,thick=1,col=blue

        tmp=lorentzian_asym(freq(cut_index1:cut_index2),result)
        tmp2=freq(cut_index1:cut_index2)
    legend,['data','fit'],linestyle=[0,3],charsize=1.5,pos=[max(tmp2)*0.7,max(tmp)*0.9],col=[0,80]

    xyouts,max(tmp2)*0.7,max(tmp)*0.95,'!8kR!I!9n!N!5 = '+strcompress(string(ceil(ell(l))),/remove_all),charsize=2,charthick=2

; here oplot the model S value of ell
    oplot,[feig(l),feig(l)],[-1,1]*1e3
    oplot,[peig1(l),peig1(l)],[-1,1]*1e3
    oplot,[peig2(l),peig2(l)],[-1,1]*1e3
    oplot,[peig3(l),peig3(l)],[-1,1]*1e3
    oplot,[peig4(l),peig4(l)],[-1,1]*1e3

    
;    for i=0,4 do oplot,[fg(i),fg(i)],[0,1e5]

    print,"Index is ",l," and the corresponding ell value is ",ell(l)
    param_array(index,0)=ell(l)
    param_array(index,1:*)=result
; no errors for now
;    errors(index,*)=errs

    index=index+1
    
endfor ; l


mkhdr,head,param_array
sxaddpar,head,'DF',df*1d6,' micro Hz',AFTER='NAXIS2'
sxaddpar,head,'DL',dell,AFTER='DF'
sxaddpar,head,'NP',peaks,' number of fitted  peaks',AFTER='DL'
sxaddpar,head,'NF',ny,' original',AFTER='NP'
sxaddpar,head,'SMOOTH',smooth_fac,' pix - smoothing factor',AFTER='NF'
sxaddpar,head,'COMMENT','ELL(0) FREQ(1:peaks)  LINE-WIDTH(peaks+1:2*peaks)  '
sxaddpar,head,'COMMENT', 'AMPLITUDE(2*peaks+1:3*peaks) ASYMMETRY(3*peaks+1:4*peaks)  NOISE(4*peaks+1) '


print," "
print,"Writing file "+ODIR+"fit_parameters.fits"
print," "
writefits,ODIR+"fit_parameters.fits",param_array,head

; this prints erros, no need for this now
;print," "
;print,"Writing file "+ODIR+"fit_parameters_errors.fits"
;print," "
;writefits,ODIR+"fit_parameters_errors.fits",errors,head

ENDED:

stop
END
