PRO power_spectra_general, DIR, filename, dx, dt


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; PURPOSE: to calculate and display the power spectra of a 3D FITS cube 
;          (x, y, time)
;
;
;  INPUT:  DIR: (string) directory where to read the filename from and
;              where to deposit the output
;          filename: (string) name of the file to read in - must be a "hslice"
;          dx: (float) spatial scale -per Mm (assume dx=dy)
;          dt: temporal resolution -seconds
;
;  OUTPUT: outputs the azimuthally averaged power spectra FITS file
;         to the specified DIR.
;
; .r /data/seismo1/schunker/IDL_PRO/azim_2d.pro
; .r /data/seismo1/schunker/IDL_PRO/imdisp 
; .r /data/seismo2/schunker/SLIM/v2.5/CODE/PRO/analysis/power_spectra_general.pro
;
; EXAMPLE:
; power_spectra_general,"/scratch/seismo1/schunker/final_csma_z300/","vz_csma.fits",1.4577, 30.
;
;
; Author: Hannah Schunker
; schunker [@] mps.mpg.de
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
!p.multi=[0,1,1,0,0]

Rs=696.0*1d8 ; cm

v=readfits(DIR+filename)

nx=n_elements(v(*,0,0))
ny=n_elements(v(0,*,0))
nz=n_elements(v(0,0,*))
onx=nx


dy=dx

vv=v

;;;;   begin   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; filter in space with a hanning window
nz=n_elements(vv(0,0,*))
han=apod(nx,ny,0.8*nx/2.,0.94*nx/2.)
print,"Apodizing"
for k=0,nz-1 do vv(*,*,k)=vv(*,*,k)*han

vv=reform(vv)
tn=n_elements(vv(0,0,*))


; make the time even 
if tn mod 2 ne 0 then begin
   vv=vv(*,*,0:tn-2)
   tn=n_elements(vv(0,0,*)) 
endif


; pad in time
print, "Padding double in time"
vvv=dblarr(nx,ny,2*tn)
vvv(*,*,tn/2:tn/2+tn-1)=vv
tn=n_elements(vvv(0,0,*))
vv=vvv


; do the fft
fv=fft(vv,/double)

; calculate the power
av=abs(fv)^2
avn=av

; shift so (kx,ky)=(0,0) is at centre pixel
av=shift(av,nx/2,ny/2,0)


; make it odd in Kx and Ky space so kx,ky=0 is at centre pixel
; this removes the Nyquist frequency
av=av(1:*,1:*,*)
nx=n_elements(av(*,0,0))
ny=n_elements(av(0,*,0))

; azimuthally average
azcube=av*0.
for i=0,tn-1 do azcube(*,*,i)=azim_2d(av(*,*,i))
av=reform(azcube(*,ny/2,*))



; define the frequency axis
df=1./(dt*tn)
freq=indgen(tn/2.+1)*df


; define the ell axis
dk=1./(dx*1d8*onx)
kvec=indgen(nx/2.+1)
ell=kvec*dk*2*!pi*696.d0*1e8
dl=ell(1)-ell(0)



; cut the spectra to size
avc=av(nx/2:*,0:tn/2)
nx=n_elements(avc(*,0))
ny=n_elements(avc(0,*))





;;;;;;;;  NORMALISE THE SPECTRA  ;;;;;;;;  
print,"   "
print,"Normalise by the mean of an elliptical area centred at ell=800 and omega=3 mHz"
print,"   "
; make an ellipse centred on 3 mHz and ell=800
; in the power spectra and find the
; mean power in there to use as the normalisation factor
cf=0.003   ; Hz
cl=800. ; ell
rmaj=200.  ; ell 
rmin=0.001     ; Hz
ellipse=avc*0
for e=0,nx-1 do for ee=0,ny-1 do if ( (ell(e)-cl)^2/rmaj^2 + (freq(ee)-cf)^2/rmin^2 le 1) then ellipse(e,ee)=1

fac=ellipse*avc

s1=avc/mean(fac(where(ellipse ne 0)))
avc=s1




;;;;   DISPLAY AND WRITE OUT 


imdisp,alog(avc > 1e-5),/axis,aspect=1,xrange=[0-dl/2.,max(ell)-dl/2.],yrange=[min(freq),max(freq)]*1e3,xtit='!8kR!9!In!N!5',ytit='frequency (mHz)'

MDIR="/data/seismo2/schunker/SLIM/EIGS/BOX_145.770Mm/"
eigxl=readfits(MDIR+"harmonic_degree.fits")*1d0
feig=readfits(MDIR+"f_eigvals.fits")/(2*!pi)*1d6   ;micro Hz
peig1=readfits(MDIR+"p1_eigvals.fits")/(2*!pi)*1d6
peig2=readfits(MDIR+"p2_eigvals.fits")/(2*!pi)*1d6
peig3=readfits(MDIR+"p3_eigvals.fits")/(2*!pi)*1d6
peig4=readfits(MDIR+"p4_eigvals.fits")/(2*!pi)*1d6
eigxl=[0,reform(eigxl)]
feig=[feig(0),reform(feig)]
peig1=[peig1(0),reform(peig1)]
peig2=[peig2(0),reform(peig2)]
peig3=[peig3(0),reform(peig3)]
peig4=[peig4(0),reform(peig4)]

oplot,eigxl,feig*1e-3
oplot,eigxl,peig1*1e-3
oplot,eigxl,peig2*1e-3
oplot,eigxl,peig3*1e-3
oplot,eigxl,peig4*1e-3
mkhdr,head,avc

sxaddpar,head,'DF',df,'Hz',AFTER='DK'
sxaddpar,head,'DL',dl
sxaddpar,head,'DK',dk*1d8,'pixel per Mm',AFTER='NAXIS3'



name="powerspectra_general.fits"
print,"Writing file "+DIR+name
writefits,DIR+name,avc,head


STOP


END
